function [Mc, Uc, Mnc, pnc, Unc] = order_selection( X )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% order selection function
% input: the observations X
% outputs:  the order Mc by assuming circular model
%           the signal subspace Uc by assuming circular model
%           the order Mnc by assuming noncircular model
%           the number of noncircular components pnc by assuming noncircular model
%           the signal subspace Unc by assuming noncircular model
%
%
% Reference:
% X.-L. Li, T. Adali, and M. Anderson, "Noncircular principal component
% analysis and its application to model selection," IEEE Trans. Signal Processing, 
% vol. 59, no. 10, pp. 4516-4528i, Oct. 2011. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[N,T] = size(X);
dlc = inf;
dlnc = inf;
DL = NaN(N,N);
for M = 0 : N-1
    for p = 0 : M
        [U, dl] = ncPCA( X, M, p );
        DL(M+1,p+1) = dl;
        
        if p==0
            if dl < dlc
                dlc = dl;
                Mc = M;
                Uc = U;
            end
        end
        
        if dl < dlnc
            dlnc = dl;
            Mnc = M;
            pnc = p;
            Unc = U;
        end
    end
end

% figure;
% plot(0:N-1, DL(:,1),'b--d')
% hold on; plot(0:N-1,diag(DL),'k--o')
% xlabel('Order')
% ylabel('Description length')
% legend('{\itp} = 0', '{\itp} = {\itM}')

function [U, dl] = ncPCA( X, M, p )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% noncircular PCA: PCA in the presence of noncircular signals
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Inputs:
% X: observed data
% M: number of signals
% p: number of noncircular signals, p<=M
% Outputs:
% U: signal subspace
% dl: description length per sample
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin < 3
    p = M;
end
if p > M
    p = M;
end

[N,T] = size(X);
for n = 1 : N
    X(n,:) = X(n,:) - mean(X(n,:));
end
hatC = X*X'/T; hatP = X*X.'/T;
[U_svd,D_svd,V_svd] = svd(X',0);
d = sqrt( diag(D_svd).^2/T ); 

% initial guess for U
U0 = V_svd(:,1:M);
U=U0;
% initial guess for Lambda
Lambda = d(1:M);
% inital guess for sigma2
sigma2 = sum(d(M+1:N).^2)/(N-M);

if p == 0   % circular model
    dl = ( M*(2*N-M)+p*(2*M-p+1)+1 )*log(T)/T/2;
    dl = dl + N*(1+log(pi)) + (N-M)*log(sigma2);
    for m = 1 : M
        dl = dl + 2*log(Lambda(m));
    end
    return;
end

% begins noncircular PCA
max_iter = 1000;
cost = zeros(max_iter,1);
last_U = U;
muU = 1e10;
min_cost = inf;
best_U = U;
last_U = U;
for iter = 1 : max_iter
    
    % update sigma2
    sigma2 = real( trace(hatC) - trace(U'*hatC*U) )/(N-M);
    % update Lambda
    Lambda = sqrt(real(diag( U'*hatC*U )));      
    % update V and K
    Q = diag(1./Lambda)*U'*hatP*conj(U)*diag(1./Lambda);
    [Vq,Dq] = eig(Q*conj(Q));
    kappaq = diag(Vq'*Q*conj(Vq));
    [tempY, I] = sort(abs(kappaq),'descend');
    K = kappaq(I(1:p));
    V = Vq(:,I(1:p));
    
    C = U*diag(Lambda.^2-sigma2)*U' + sigma2*eye(N);
    P = U*diag(Lambda)*V*diag(K)*V.'*diag(Lambda)*U.';
    uC = [C, P; conj(P), conj(C)];
    uhatC = [hatC, hatP; conj(hatP), conj(hatC)];
    cost(iter) = 0.5*log(det(uC)) + 0.5*trace( inv(uC)*uhatC );
    cost(iter) = real(cost(iter));
    
    % update U
    gradU = zeros(size(U));
    gradU = -hatC*U/sigma2 + hatC*U*diag(1./Lambda.^2);
    gradU = gradU + hatC*U*diag(1./Lambda)*V*diag( K.*conj(K)./(1-K.*conj(K)) )*V'*diag(1./Lambda);
    gradU = gradU - hatP*conj(U)*diag(1./Lambda)*conj(V)*diag( conj(K)./(1-K.*conj(K)) )*V'*diag(1./Lambda);
    gradU = gradU/norm(gradU);
    U = U - muU*gradU;
    U = U*inv(sqrtm(U'*U));
    
    tempC = U'*hatC*U;
    [tempV, tempD] = eig(tempC);
    U = U*tempV;

    %%%%%%%%%%%%%%%%%
    if 1-min(diag(abs(U'*best_U))) < 1e-10
        break;
    end

    if cost(iter) < min_cost 
        min_cost = cost(iter);
        best_U = last_U;
        last_U = U;
    else
        U = best_U;
        last_U = U;
        muU = muU/10;
        if muU < 1e-10
            break;
        end
    end
       
end

dl = ( M*(2*N-M)+p*(2*M-p+1)+1 )*log(T)/T/2;
dl = dl + N*(1+log(pi)) + (N-M)*log(sigma2);
for m = 1 : M
    dl = dl + 2*log(Lambda(m));
end
for el = 1 : p
    dl = dl + log( 1-(abs(K(el)))^2 )/2;
end

% figure; plot(real(cost(1:iter)))
% xlabel('Number of iterations')
% ylabel('Cost')